package de.tudresden.inf.st.ros2rag.receiverstub;

import com.google.protobuf.InvalidProtocolBufferException;
import config.Robotconfig.RobotConfig;
import de.tudresden.inf.st.ros2rag.starter.ast.MqttUpdater;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import panda.Linkstate.PandaLinkState;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class Main {

  private static final Logger logger = LogManager.getLogger(Main.class);

  public static void main(String[] args) throws Exception {
    String jointTopic, configTopic;
    if (args.length < 2) {
      jointTopic = "robot/joint1";
      configTopic = "robot/config";
    } else {
      jointTopic = args[0];
      configTopic = args[1];
    }

    final Lock finishLock = new ReentrantLock();
    final Condition finishCondition = finishLock.newCondition();

    MqttUpdater receiver = new MqttUpdater("receiver stub");
    receiver.setHost("localhost", 1883);
    receiver.waitUntilReady(2, TimeUnit.SECONDS);
    receiver.newConnection(configTopic, bytes -> {
      try {
        logger.debug("Got a config message, parsing ...");
        RobotConfig robotConfig = RobotConfig.parseFrom(bytes);
        logger.info("robotConfig: speed = {}, loopTrajectory = {}, planningMode = {}",
            robotConfig.getSpeed(),
            robotConfig.getLoopTrajectory(),
            robotConfig.getPlanningMode().toString());
      } catch (InvalidProtocolBufferException e) {
        e.printStackTrace();
      }
    });
    receiver.newConnection(jointTopic, bytes -> {
      try {
        logger.debug("Got a joint message, parsing ...");
        PandaLinkState pls = PandaLinkState.parseFrom(bytes);
        PandaLinkState.Position tmpPosition = pls.getPos();
        PandaLinkState.Orientation tmpOrientation = pls.getOrient();
        PandaLinkState.TwistLinear tmpTwistLinear = pls.getTl();
        PandaLinkState.TwistAngular tmpTwistAngular = pls.getTa();
        logger.info("{}: pos({},{},{}), orient({},{},{},{})," +
                " twist-linear({},{},{}), twist-angular({},{},{})",
            pls.getName(),
            tmpPosition.getPositionX(),
            tmpPosition.getPositionY(),
            tmpPosition.getPositionZ(),
            tmpOrientation.getOrientationX(),
            tmpOrientation.getOrientationY(),
            tmpOrientation.getOrientationZ(),
            tmpOrientation.getOrientationW(),
            tmpTwistLinear.getTwistLinearX(),
            tmpTwistLinear.getTwistLinearY(),
            tmpTwistLinear.getTwistLinearZ(),
            tmpTwistAngular.getTwistAngularX(),
            tmpTwistAngular.getTwistAngularY(),
            tmpTwistAngular.getTwistAngularZ());
      } catch (InvalidProtocolBufferException e) {
        e.printStackTrace();
      }
    });
    receiver.newConnection("components", bytes -> {
      String message = new String(bytes);
      logger.info("Components: {}", message);
    });
    receiver.newConnection("receiver", bytes -> {
      String message = new String(bytes);
      if (message.equals("exit")) {
        try {
          finishLock.lock();
          finishCondition.signal();
        } finally {
          finishLock.unlock();
        }
      }
    });
    try {
      finishLock.lock();
      finishCondition.await();
    } finally {
      finishLock.unlock();
    }
    receiver.close();
    Runtime.getRuntime().addShutdownHook(new Thread(receiver::close));
  }
}
